
package eu.dnetlib.dhp.oa.graph.dump;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;

import eu.dnetlib.dhp.common.HdfsSupport;
import eu.dnetlib.dhp.oa.graph.dump.community.CommunityMap;
import eu.dnetlib.dhp.oa.graph.dump.complete.Constants;
import eu.dnetlib.dhp.oa.model.graph.GraphResult;
import eu.dnetlib.dhp.oa.model.graph.Relation;
import eu.dnetlib.dhp.oa.model.graph.ResearchCommunity;
import eu.dnetlib.dhp.utils.DHPUtils;
import eu.dnetlib.dhp.utils.ISLookupClientFactory;
import eu.dnetlib.enabling.is.lookup.rmi.ISLookUpService;
import scala.Tuple2;

public class Utils {
	public static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

	private Utils() {
	}

	public static void removeOutputDir(SparkSession spark, String path) {
		HdfsSupport.remove(path, spark.sparkContext().hadoopConfiguration());
	}

	public static <R> Dataset<R> readPath(
		SparkSession spark, String inputPath, Class<R> clazz) {
		return spark
			.read()
			.textFile(inputPath)
			.map((MapFunction<String, R>) value -> OBJECT_MAPPER.readValue(value, clazz), Encoders.bean(clazz));
	}

	public static ISLookUpService getIsLookUpService(String isLookUpUrl) {
		return ISLookupClientFactory.getLookUpService(isLookUpUrl);
	}

	public static String getContextId(String id) {

		return String
			.format(
				"%s|%s::%s", Constants.CONTEXT_ID, Constants.CONTEXT_NS_PREFIX,
				DHPUtils.md5(id));
	}

	public static CommunityMap getCommunityMap(SparkSession spark, String communityMapPath) {

		return new Gson().fromJson(spark.read().textFile(communityMapPath).collectAsList().get(0), CommunityMap.class);

	}

	public static CommunityMap readCommunityMap(FileSystem fileSystem, String communityMapPath) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(fileSystem.open(new Path(communityMapPath))));
		StringBuilder sb = new StringBuilder();
		try {
			String line;
			while ((line = br.readLine()) != null) {
				sb.append(line);
			}
		} finally {
			br.close();

		}

		return new Gson().fromJson(sb.toString(), CommunityMap.class);
	}

	public static Dataset<String> getEntitiesId(SparkSession spark, String inputPath) {
		Dataset<String> dumpedIds = Utils
			.readPath(spark, inputPath + "/publication", GraphResult.class)
			.map((MapFunction<GraphResult, String>) r -> r.getId(), Encoders.STRING())
			.union(
				Utils
					.readPath(spark, inputPath + "/dataset", GraphResult.class)
					.map((MapFunction<GraphResult, String>) r -> r.getId(), Encoders.STRING()))
			.union(
				Utils
					.readPath(spark, inputPath + "/software", GraphResult.class)
					.map((MapFunction<GraphResult, String>) r -> r.getId(), Encoders.STRING()))
			.union(
				Utils
					.readPath(spark, inputPath + "/otherresearchproduct", GraphResult.class)
					.map((MapFunction<GraphResult, String>) r -> r.getId(), Encoders.STRING()))
			.union(
				Utils
					.readPath(spark, inputPath + "/organization", eu.dnetlib.dhp.oa.model.graph.Organization.class)
					.map(
						(MapFunction<eu.dnetlib.dhp.oa.model.graph.Organization, String>) o -> o.getId(),
						Encoders.STRING()))
			.union(
				Utils
					.readPath(spark, inputPath + "/project", eu.dnetlib.dhp.oa.model.graph.Project.class)
					.map(
						(MapFunction<eu.dnetlib.dhp.oa.model.graph.Project, String>) o -> o.getId(), Encoders.STRING()))
			.union(
				Utils
					.readPath(spark, inputPath + "/datasource", eu.dnetlib.dhp.oa.model.graph.Datasource.class)
					.map(
						(MapFunction<eu.dnetlib.dhp.oa.model.graph.Datasource, String>) o -> o.getId(),
						Encoders.STRING()))
			.union(
				Utils
					.readPath(spark, inputPath + "/communities_infrastructures", ResearchCommunity.class)
					.map((MapFunction<ResearchCommunity, String>) c -> c.getId(), Encoders.STRING()));
		return dumpedIds;
	}

	public static Dataset<Relation> getValidRelations(SparkSession spark, Dataset<Relation> relations,
		Dataset<String> entitiesIds) {
		Dataset<Tuple2<String, Relation>> relationSource = relations
			.map(
				(MapFunction<Relation, Tuple2<String, Relation>>) r -> new Tuple2<>(r.getSource().getId(), r),
				Encoders.tuple(Encoders.STRING(), Encoders.bean(Relation.class)));

		Dataset<Tuple2<String, Relation>> relJoinSource = relationSource
			.joinWith(entitiesIds, relationSource.col("_1").equalTo(entitiesIds.col("value")))
			.map(
				(MapFunction<Tuple2<Tuple2<String, Relation>, String>, Tuple2<String, Relation>>) t2 -> new Tuple2<>(
					t2._1()._2().getTarget().getId(), t2._1()._2()),
				Encoders.tuple(Encoders.STRING(), Encoders.bean(Relation.class)));

		return relJoinSource
			.joinWith(entitiesIds, relJoinSource.col("_1").equalTo(entitiesIds.col("value")))
			.map(
				(MapFunction<Tuple2<Tuple2<String, Relation>, String>, Relation>) t2 -> t2._1()._2(),
				Encoders.bean(Relation.class));
	}

}
